# GradeExpectations 4.0.0

### *- For the Grader Good*

**GradeExpectations** is a webscraper and grade monitor designed for the [ProgressBook](https://paccess.mveca.org) grading program.  

## License

GradeExpectations © 2025 by Warren James is licensed under Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International. To view a copy of this license, visit [creativecommons.org/licenses/by-nc-sa/4.0](https://creativecommons.org/licenses/by-nc-sa/4.0/).

## Installation

### 1. Run `schedule.bat` to Tell **GradeExpectations** When to Run

**GradeExpectations** will use the **Windows Task Scheduler** to create a daily routine for running `grade.exe`.  
When it asks for the time it should run at daily, the provided time should be in the **HH:MM** format.  
For example, if you want **GradeExpectations** to run at **5:42 pm** every day, you would enter **17:42**, the military time for 5:42 pm.  
If you wanted **3:15 am**, you would enter **03:15**, including the leading zero.  
After you have told **GradeExpectations** when to run, it will ask whether you want to create a desktop shortcut for `view.exe`, enter "y" or "n".

### 2. Provide Login Information

To link **GradeExpectations** with ProgressBook, open `login.json` and enter a parental account's login credentials like so, making sure that the login details are inside the quotes:

```json
{
    "username": "MyParentAccount",
    "password": "Password123",
    "loginURL": "https://paccess.mveca.org/district/DISTRICT"
}
```

The "loginURL" parameter is the link that opens the ProgressBook login page for a given district.  
To find yours:

1. Go to [paccess.mveca.org/district](https://paccess.mveca.org/district).
2. Find and right click on your district.
3. Click "Copy link address".
4. Paste the address between the double quotes after the "loginURL" parameter in `login.json`.

## How It Works

**GradeExpectations** uses the `selenium` library to auto login to ProgressBook using the details provided in the `login.json` file.  
For each student on the provided parental account, it cycles to the grades category and saves each students' grades to their own csv file using `pandas`.  

### How to Edit the **GradeExpectations** Task

To edit the **GradeExpectations** task, you can:

- Open the **Windows Task Scheduler** and manually change the task named "GradeExpectations".
- Run `schedule.bat` again to just change the start time.

### An Import Note

The **GradeExpectations** task can only run when the computer is **turned on** and **the user who created the task is logged in.**  
So, make sure the time you enter in **Step 1** of Installation is a time when **that user will always be logged onto the computer.**  
The least intrusive way to achieve this is leaving the computer logged on overnight and setting the run time to something like **23:30**.

## Viewing the Grades

1. Run `view.exe` or Its Shortcut.
2. Select a Grade CSV to View.

### Understanding the Grade Graph

- Graph: The student's grades over time.
- X-axis: The date the grades were recorded.
    - Major lines: Mondays
    - Minor lines: Days
- Y-axis: The grade for the given course at that moment in time.
    - Major lines: Multiples of 5%
    - Minor lines: Multiples of 1%
- Color Bands: The letter grade assigned to each percent range.
- Title: If the Grade CSV is under the standard directories, `Grades/YYYY-YYYY/Quarter X/STUDENT NAME.csv`, the title will be of the form:
```
        GradeExpectations       
YYYY-YYYY Quarter X STUDENT NAME
```

### Seeing Your GPA

To see your GPA instead of your grades, change the **view** parameter in `view.json` to **"GPA"** or **"Ask"**.  
Upon opening the graph, there are four types of GPAs by default:

1. **gpa**, Grade Point Average: The most simple type. It only looks at the letter grade for the course without the plus (+) or minus (-). An "A" is a 4.0; "B", 3.0; "C", 2.0; and so on.
2. **gpa+**, Signed Grade Point Average: The same as **gpa** but a bit more nuanced. It takes into account the sign of the grade: "A+" being 4.0, "A" being 3.9, "A-" being 3.7, "B+" being 3.3, "B" being 3.0, "B-" being 2.7, and so on.
3. **wgpa**, Weighted Grade Point Average: The same as **gpa** but adds a bonus to each course. Taken from the **courseWeights** parameter, the difference between the provided course weight and 4.0 is added to the final GPA.
4. **wgpa+**, Weighted Signed Grade Point Average: The combination of **gpa+** and **wgpa**.

### Understanding the GPA Graph

- Y-axis: The GPA at that moment in time.
    - Major lines: Multiples of 0.10
    - Minor lines: Multiples of 0.02
- Color Bands: The integer assigned to each GPA range.

### Configuring `view.json`

- **view**
    - The data to be viewed.
    - Values
        - "Grades"
        - "GPA"
        - "Ask": Before choosing the Grade CSV, you will be asked if you want to view the Grades over time or GPA over time. Type "g" or "grade" for Grades or "G" or "gpa" for GPA.
- **defaultDirectory**
    - The directory opened when choosing a Grade CSV.
    - Values
        - "Latest": Opens the most recent Quarter folder in Grades.
        - "YYYY-YYYY/Quarter X/": Opens the provided relative path inside the Grades folder.
        - "/Absolute/Path/To/Grade/CSV/": Opens the provided absolute path to a Grade CSV directory.
- **courseWeights**
    - The weights uses in weighted GPA calculation.
    - Format: `{ "Full Course Name": value }`
    - To find the full course name, you can either copy the name from ProgressBook or copy from the first line in a Grade CSV.
    - Values
        - 4.0: Nothing is added in the GPA calculation.
        - 4.5: 0.5 is added to the course in the calculation.
        - 5.0: 1.0 is added to the course in the calculation.
    - Yours could could look something like this:
    ```json
    "courseWeights": {
        "Geometry 10": 4.0,
        "Scholarship English 10": 4.5,
        "Honors Biology 10": 5.0
    }
    ```
- **roundGrade**
    - Determines if the grade is rounded before converting it to a GPA, so if a 90 is a 4.0, then so is an 89.5.
    - Note this doesn't round the grade if the view is Grades.
- **gradeToGPA**
    - The conversion table used by **gpa** and **wgpa**.
    - Change if your school's calculations are different.
- **gradeToSignedGPA**
    - The conversion table used by **gpa+** and **wgpa+**.
    - Change if your school's calculations are different.
- **gradeLimits**
    - The min and max of the y-axis when viewing Grades over time.
    - Values
        - number: The value to be used to bound the y-axis unless a datapoint exceeds said value.
        - null: The min or max will be the min or max of the dataset.
- **gpaLimits**
    - Same as **gradeLimits** but for the GPA view.
- **windowColors**
    - The hex values used to color the window.
- **courseColors**
    - The hex values used to color the course names in the legend and the lines on the grade.
    - If you have more than eight courses, add hex colors to the list.
- **gpaColors**
    - The hex values used to color the four types of GPA in the legend and on the grade.
- **gradeBands**
    - The bands of color used to denote the grade letter associated with the course grade.
    - If you would prefer a target or goal band instead, just customize this variable.
    - Values
        - null: To be used at the start of the end of the bands. Refers to either positive or negative infinity.
- **gpaBands**
    - Same as **gradeBands** but for the GPA view.
To change the **windowColors**, **courseColors**, or **gradeColors**, enter new hex values inside `view.json`.  

### Problems

- If no classes are displayed, come back in a few days when some data has been recorded.
- If you happen to have more than eight courses, you should add more colors to the **courseColors** list in `view.json`. If no more colors are added, the graph will reuse colors to populate the legend.  

## The File Structure

When the `grade.exe` program is first run, it will create a Grades folder.  
Inside, it will create another folder for the given school year and a third for the given quarter.  
Per quarter, there is one csv per student.  

Yours should look something like this:

```
GradeExpectations
├── _internal
├── Grades
│   └── 2023-2024
│       ├── Quarter 1
│       │   ├── Jane Doe.csv
│       │   └── John Doe.csv
│       ├── Quarter 2
│       │   ├── Jane Doe.csv
│       │   └── John Doe.csv
│       ├── Quarter 3
│       │   ├── Jane Doe.csv
│       │   └── John Doe.csv
│       └── Quarter 4
│           ├── Jane Doe.csv
│           └── John Doe.csv
├── README.md
├── Resources
│   ├── GradeExpectations.ico
│   ├── logo.txt
│   ├── text.ttf
│   └── title.ttf
├── grade.exe
├── login.json
├── schedule.bat
├── view.exe
└── view.json
```

## Version History

- 4.0.0
    - Added GPA to view.py
    - Organized with load.py
    - Converted py's to exe
    - Enhanced logging
    - More view.json options
    - Removed all bat files
    - Created schedule.bat
- 3.0.1
    - Removed nominal reference error and print statement
- 3.0.0
    - Added defaultDirectory to view.json
    - Created data propagation in view.py
- 2.0.0
    - Added loginURL to login.json
- 1.1.0
    - Updated view.py
    - Added view.json
    - Custom fonts
    - Color bands
    - Constant minor and major steps
- 1.0.0
    - Origin